/* Copyright(C) 2014 Cobac.Net All rights reserved. */
/* MiZedBoardpj*/

module syncgen(
    input               clk,
    input               reset,
    output              pck,
    output  reg         vga_hs,
    output  reg         vga_vs,
    output  reg [9:0]   hcnt,
    output  reg [9:0]   vcnt
);

/* VGA(640~480)p[^ */
localparam HSC = 10'd800;
localparam HFP = 10'd16;
localparam HPW = 10'd96;
localparam HBP = 10'd48;

localparam VSC = 10'd525;
localparam VFP = 10'd10;
localparam VPW = 10'd2;
localparam VBP = 10'd33;

/* PixelNbN쐬 */
reg [1:0] cnt;

always @( posedge clk, posedge reset ) begin
    if ( reset )
        cnt <= 2'h0;
    else
        cnt <= cnt + 2'h1;
end

assign pck = cnt[1];

/* JE^ */
wire hcntend = (hcnt==HSC-10'h001);

always @( posedge pck ) begin
    if ( reset )
        hcnt <= 10'h000;
    else if ( hcntend )
        hcnt <= 10'h000;
    else
        hcnt <= hcnt + 10'h001;
end

/* JE^ */
always @( posedge pck ) begin
    if ( reset )
        vcnt <= 10'h000;
    else if ( hcntend ) begin
        if ( vcnt==VSC-10'h001 )
            vcnt <= 10'h000;
        else
            vcnt <= vcnt + 10'h001;
    end
end

/* M */
wire [9:0] hsstart = HFP - 10'h001;
wire [9:0] hsend   = HFP + HPW - 10'h001;
wire [9:0] vsstart = VFP;
wire [9:0] vsend   = VFP + VPW;

always @( posedge pck ) begin
    if ( reset )
        vga_hs <= 1'b1;
    else if ( hcnt==hsstart )
        vga_hs <= 1'b0;
    else if ( hcnt==hsend )
        vga_hs <= 1'b1;
end

always @( posedge pck ) begin
    if ( reset )
        vga_vs <= 1'b1;
    else if ( hcnt==hsstart ) begin
        if ( vcnt==vsstart )
            vga_vs <= 1'b0;
        else if ( vcnt==vsend )
            vga_vs <= 1'b1;
    end
end

endmodule
